@@ -14,13 +14,15 @@ def empty():
     return dict(inputs=[], outputs=[], global_options=None)
 
 
-def check_url(url, nodata=True):
+def check_url(url, nodata=True, nofileobj=False):
     """Analyze url argument for non-url input
 
     :param url: url argument string or data or file
     :type url: str, bytes-like object, numpy.ndarray, or file-like object
     :param nodata: True to raise exception if url is a bytes-like object, default to True
     :type nodata: bool, optional
+    :param nofileobj: True to raise exception if url is a file-like object, default to False
+    :type nofileobj: bool, optional
     :return: url string, file object, and data object
     :rtype: tuple<str, file-like object or None, bytes-like object or None>
     """
@@ -38,6 +40,8 @@ def check_url(url, nodata=True):
             data = url
             url = "-"
         elif hasmethod(url, "fileno"):
+            if nofileobj:
+                raise ValueError("File-like object cannot be specified as url.")
             fileobj = url
             url = "-"
 
@@ -153,46 +157,7 @@ def finalize_video_read_opts(
             _, ncomp, dtype, remove_alpha = utils.get_pixel_config(pix_fmt_in, pix_fmt)
 
     # set up basic video filter if specified
-    fopts = {
-        name: outopts.pop(name)
-        for name in (
-            "fill_color",
-            "crop",
-            "flip",
-            "transpose",
-        )
-        if name in outopts
-    }
-
-    # check if output needs to be scaled
-    scale = outopts.get("s", None)
-    do_scale = scale is not None
-    if do_scale:
-        try:
-            m = re.match(r"(\d+)x(\d+)", scale)
-            scale = (int(m[1]), int(m[2]))
-        except:
-            pass
-        try:
-            do_scale = len(scale) > 2 or [scale[0] <= 0 or scale[1] <= 0]
-        except:
-            do_scale = False
-
-    nfo = len(fopts)
-    if (nfo and (nfo > 1 or "fill_color" not in fopts)) or remove_alpha or do_scale:
-        if do_scale:
-            fopts["scale"] = scale
-            del outopts["s"]
-
-        if remove_alpha:
-            fopts["remove_alpha"] = True
-
-        if "vf" in outopts:
-            raise ValueError(
-                f"cannot specify `vf` and video basic filter options {tuple(fopts.keys())}"
-            )
-
-        outopts["vf"] = video_basic_filter(**fopts)
+    build_basic_vf(args, remove_alpha, ofile)
 
     outopts["f"] = "rawvideo"
 
@@ -211,6 +176,92 @@ def finalize_video_read_opts(
     return dtype, shape, r
 
 
+def check_alpha_change(args, dir=None, ifile=0, ofile=0):
+    # check removal of alpha channel
+    inopts = args["inputs"][ifile][1]
+    outopts = args["outputs"][ofile][1]
+    if inopts is None or outopts is None:
+        return None if dir is None else False  # indeterminable
+    return utils.alpha_change(inopts.get("pix_fmt", None), outopts.get("pix_fmt", None))
+
+
+def build_basic_vf(args, remove_alpha=None, ofile=0):
+    """convert basic VF options to vf option
+
+    :param args: FFmpeg dict
+    :type args: dict
+    :param remove_alpha: True to add overlay filter to add a background color, defaults to None
+    :                    This argument would be ignored if `'remove_alpha'` key is defined in `'args'`.
+    :type remove_alpha: bool, optional
+    :param ofile: output file id, defaults to 0
+    :type ofile: int, optional
+    :param force: True to chain basic vf to existing vf, defaults to False
+    :type force: bool, optional
+    """
+
+    # get output opts, nothing to do if no option set
+    outopts = args["outputs"][ofile][1]
+    if outopts is None:
+        return
+
+    # extract the options
+    fopts = {
+        name: outopts.pop(name)
+        for name in (
+            "fill_color",
+            "crop",
+            "flip",
+            "transpose",
+            "square_pixels",
+            "remove_alpha",
+        )
+        if name in outopts
+    }
+
+    force = outopts.pop("_force_basic_vf", False)
+
+    # check if output needs to be scaled
+    scale = outopts.get("s", None)
+    do_scale = scale is not None
+    if do_scale:
+        try:
+            m = re.match(r"(\d+)x(\d+)", scale)
+            scale = (int(m[1]), int(m[2]))
+        except:
+            pass
+        try:
+            do_scale = len(scale) > 2 or (scale[0] <= 0 or scale[1] <= 0)
+        except:
+            do_scale = False
+
+    nfo = len(fopts)
+    if (nfo and (nfo > 1 or "fill_color" not in fopts)) or remove_alpha or do_scale:
+        if do_scale:
+            fopts["scale"] = scale
+            del outopts["s"]
+
+        if remove_alpha and "remove_alpha" not in fopts:
+            fopts["remove_alpha"] = True
+
+        bvf = video_basic_filter(**fopts)
+        vf = outopts.get("vf", None)
+        if vf:
+            if not force:
+                raise ValueError(
+                    f"cannot specify `vf` and video basic filter options {tuple(fopts.keys())}"
+                )
+            if isinstance(vf, str):
+                vf += f",{bvf}"
+            else:
+                # TODO support for FilterGraph
+                raise ValueError(
+                    f"only str filtergraph description is currently supported"
+                )
+            outopts["vf"] = vf
+        else:
+            outopts["vf"] = bvf
+
+
 def finalize_audio_read_opts(
     args, sample_fmt_in=None, ac_in=None, ar_in=None, ofile=0, ifile=0
 ):
@@ -384,3 +435,52 @@ def clear_loglevel(args):
         logging.warn("loglevel option is cleared by ffmpegio")
     except:
         pass
+
+
+def finalize_media_read_opts(args):
+    """finalize multiple-input media reader setup
+
+    :param args: FFmpeg dict
+    :type args: dict
+    :return: use_ya8 flag - True to expect 16-bpc pixel to be ya8 pix_fmt instead of gray16le pix_fmt
+    :rtype: bool
+
+    - assumes options dict of the first output is already present
+    - insert `pix_fmt='rgb24'` and `sample_fmt='sa16le'` options if these options are not assigned
+    - check for the use of both 'gray16le' and 'ya8', and returns True if need to use 'ya8'
+    - set f=avi and vcodec=rawvideo
+    - set acodecs according to sample_fmts
+
+    """
+
+    # get output options, create new
+    options = args["outputs"][0][1]
+
+    # check to make sure all pixel and sample formats are supported
+    gray16le = ya8 = 0
+    for k in utils.find_stream_options(options, "pix_fmt"):
+        v = options[k]
+        utils.get_video_format(v)
+        if v == "gray16le":
+            gray16le += 1
+        elif v == "ya8":
+            ya8 += 1
+    if gray16le and ya8:
+        raise ValueError("gray16le and ya8 pix_fmts cannot be mixed.")
+
+    # if pix_fmt and sample_fmt not specified, set defaults
+    # user can conditionally override these by stream-specific option
+    if "pix_fmt" not in options:
+        options["pix_fmt"] = "rgb24"
+    if "sample_fmt" not in options:
+        options["sample_fmt"] = "s16"
+
+    # add output formats and codecs
+    options["f"] = "avi"
+    options["c:v"] = "rawvideo"
+
+    # add audio codec
+    for k in utils.find_stream_options(options, "sample_fmt"):
+        options[f"c:a" + k[10:]] = utils.get_audio_format(options[k])[0]
+
+    return ya8 > 0
