@@ -48,7 +48,7 @@ def _run_read(*args, shape=None, pix_fmt_in=None, s_in=None, show_log=None, **kw
             *args,
             dtype=dtype,
             shape=shape,
-            capture_log=False if show_log else True,
+            capture_log=None if show_log else False,
             **kwargs,
         )
         if out.returncode:
@@ -153,8 +153,6 @@ def read(url, show_log=None, **options):
     option which is an alias of `start` standard option.
     """
 
-    pix_fmt = options.get("pix_fmt", None)
-
     # get pix_fmt of the input file only if needed
     if "pix_fmt_in" not in options:
         info = probe.video_streams_basic(url, 0)[0]
@@ -213,12 +211,14 @@ def write(url, data, overwrite=None, show_log=None, **options):
     outopts = configure.add_url(ffmpeg_args, "output", url, options)[1][1]
     outopts["frames:v"] = 1
 
+    configure.build_basic_vf(ffmpeg_args, configure.check_alpha_change(ffmpeg_args, -1))
+
     ffmpegprocess.run(
         ffmpeg_args,
         input=data,
         stdout=stdout,
         overwrite=overwrite,
-        capture_log=False if show_log else None,
+        capture_log=None if show_log else False,
     )
 
 
