@@ -1,8 +1,20 @@
-from . import ffmpegprocess, configure, utils
+from . import ffmpegprocess, configure, utils, probe
 from .utils.log import FFmpegError
 
+__all__ = ["transcode"]
 
-def transcode(input_url, output_url, progress=None, overwrite=None, show_log=None, **options):
+
+def transcode(
+    input_url,
+    output_url,
+    progress=None,
+    overwrite=None,
+    show_log=None,
+    two_pass=False,
+    pass1_omits=None,
+    pass1_extras=None,
+    **options
+):
     """Transcode a media file to another format/encoding
 
     :param input_url: url/path of the input media file
@@ -18,6 +30,13 @@ def transcode(input_url, output_url, progress=None, overwrite=None, show_log=Non
                      defaults to None (no show/capture)
                      Ignored if stream format must be retrieved automatically.
     :type show_log: bool, optional
+    :param two_pass: True to encode in 2-pass
+    :param pass1_omits: list of output arguments to ignore in pass 1, defaults to
+                        None (removes 'c:a' or 'acodec')
+    :type pass1_omits: seq(str), optional
+    :param pass1_extras: list of additional output arguments to include in pass 1,
+                         defaults to None (add 'an' if `pass1_omits` also None)
+    :type pass1_extras: dict(int:dict(str)), optional
     :param \\**options: FFmpeg options. For output and global options, use FFmpeg
                         option names as is. For input options, prepend "input_" to
                         the option name. For example, input_r=2000 to force the
@@ -38,14 +57,34 @@ def transcode(input_url, output_url, progress=None, overwrite=None, show_log=Non
     configure.add_url(args, "input", input_url, input_options)[1][1]
     configure.add_url(args, "output", output_url, options)
 
-    pout = ffmpegprocess.run(
+    # if output pix_fmt defined, get input pix_fmt to check for transparency change
+    # TODO : stream spec?
+    pix_fmt = options.get("pix_fmt", None)
+    pix_fmt_in = (
+        probe.video_streams_basic(input_url, 0)[0]["pix_fmt"] if pix_fmt else None
+    )
+
+    # convert basic VF options to vf option
+    configure.build_basic_vf(args, utils.alpha_change(pix_fmt_in, pix_fmt, -1))
+
+    kwargs = (
+        {
+            "pass1_omits": None if pass1_omits is None else [pass1_omits],
+            "pass1_extras": None if pass1_extras is None else [pass1_extras],
+        }
+        if two_pass
+        else {}
+    )
+
+    pout = (ffmpegprocess.run_two_pass if two_pass else ffmpegprocess.run)(
         args,
         progress=progress,
         overwrite=overwrite,
-        capture_log=False if show_log else True,
+        capture_log=None if show_log else False,
         stdin=stdin,
         stdout=stdout,
         input=input,
+        **kwargs,
     )
     if pout.returncode:
         raise FFmpegError(pout.stderr, show_log)
