@@ -2,23 +2,16 @@ import re, fractions
 import numpy as np
 from .. import caps
 
-# fmt:off
-_filter_video_srcs = ("allrgb", "allyuv", "buffer", "cellauto", "color",
-    "coreimagesrc", "frei0r_src", "gradients", "haldclutsrc", "life", 
-    "mandelbrot", "mptestsrc", "nullsrc", "openclsrc", "pal100bars", 
-    "pal75bars",  "rgbtestsrc", "sierpinski", "smptebars", "smptehdbars",
-    "testsrc", "testsrc2", "yuvtestsrc",
-)
-_filter_video_snks = ("buffersink", "nullsink")
-_filter_audio_srcs = ("abuffer", "aevalsrc", "afirsrc", "anullsrc", 
-    "flite", "anoisesrc", "hilbert", "sinc", "sine")
-_filter_audio_snks = ("abuffersink", "anullsink")
-# fmt:on
 
-
-def parse_spec_stream(spec):
+def parse_spec_stream(spec,file_index=False):
     if isinstance(spec, str):
         out = {}
+        if file_index:
+            m = re.match(r'(\d+):')
+            if m:
+                out['file_index'] = int(m[1])
+                spec = spec[m.end():]
+                
         while len(spec):
             if spec.startswith("p:"):
                 _, v, *r = spec.split(":", 2)
@@ -212,6 +205,32 @@ def get_pixel_config(input_pix_fmt, pix_fmt=None):
     )
 
 
+def alpha_change(input_pix_fmt, output_pix_fmt, dir=None):
+    """get best pixel configuration to read video data in specified pixel format
+
+    :param input_pix_fmt: input pixel format
+    :type input_pix_fmt: str
+    :param output_pix_fmt: output pixel format
+    :type output_pix_fmt: str, optional
+    :param dir: specify the change direction for boolean answer, defaults to None
+    :type dir: int, optional
+    :return: dir None: 0 if no change, 1 if alpha added, -1 if alpha removed, None if indeterminable
+             dir int: True if changes in the specified direction or False
+    :rtype: bool, int, None
+
+    """
+    if input_pix_fmt is None or output_pix_fmt is None:
+        return None if dir is None else False
+    n_in = caps.pix_fmts()[input_pix_fmt]["nb_components"]
+    n_out = caps.pix_fmts()[output_pix_fmt]["nb_components"]
+    d = (n_in % 2) - (n_out % 2)
+    return (
+        d
+        if dir is None
+        else (d > 0) == (dir > 0) or (d < 0) == (dir < 0) or (d == 0 and dir == 0)
+    )
+
+
 def get_video_format(fmt):
     """get video pixel format
 
@@ -333,9 +352,11 @@ def get_audio_format(fmt, return_format=False):
 
     # byteorder = "be" if sys.byteorder == "big" else "le"
     if isinstance(fmt, str):
-        out = formats.get(fmt, formats["s16"])
+        try:
+            out = formats[fmt]
+        except:
+            raise ValueError(f"{fmt} is not a valid raw audio sample_fmt")
         return (out[0][4:], out[1]) if return_format else out
-
     else:
         try:
             return next(((v[0], k) for k, v in formats.items() if v[1] == fmt))
@@ -343,6 +364,47 @@ def get_audio_format(fmt, return_format=False):
             raise ValueError(f"incompatible numpy dtype used: {fmt}")
 
 
+def guess_audio_format(data=None):
+    """get audio format
+
+    :param data: data array or a tuple of non-temporal shape and dtype of the data
+    :type data: numpy.ndarray or (numpy.dtype, seq of ints)
+    :return: tuple of # of channels and sample_fmt
+    :rtype: tuple(int,str)
+
+    ```
+        X = np.ones((1000,2),np.int16)
+        ac, sample_fmt = guess_audio_format(X)
+        # => ac=2, sample_fmt='s16'
+
+        # the same result can be obtained by
+        ac, sample_fmt = guess_audio_format((X.shape,X.dtype))
+    """
+
+    try:
+        dtype = np.dtype(data.dtype)
+        shape = data.shape
+        ndim = data.ndim
+    except:
+        try:
+            shape, dtype = data
+            ndim = len(shape)
+            dtype = np.dtype(dtype)
+        except:
+            raise ValueError(
+                "invalid input argument: must be either numpy.array or (shape, dtype) sequence"
+            )
+
+    if ndim < 1 or ndim > 2:
+        raise ValueError(
+            f"invalid audio data dimension: data shape must be must be 1d or 2d"
+        )
+
+    sample_fmt = get_audio_format(dtype)
+
+    return shape[-1], sample_fmt
+
+
 def array_to_video_input(
     rate,
     data=None,
@@ -464,11 +526,14 @@ def array_to_audio_input(
 
 def parse_video_size(expr):
 
-    m = re.match(r"(\d+)x(\d+)", expr)
-    if m:
-        return (int(m[1]), int(m[2]))
+    if isinstance(expr, str):
+        m = re.match(r"(\d+)x(\d+)", expr)
+        if m:
+            return (int(m[1]), int(m[2]))
 
-    return caps.video_size_presets[expr]
+        return caps.video_size_presets[expr]
+    else:
+        return expr
 
 
 def parse_frame_rate(expr):
@@ -581,7 +646,30 @@ def parse_time_duration(expr):
     return expr
 
 
+def find_stream_options(options, name):
+    """find option keys, which may be stream-specific
+
+    :param options: source option dict (content will be modified)
+    :type options: dict
+    :param suffix: matching suffix
+    :type suffix: str
+    :return: popped options
+    :rtype: dict
+    """
+
+    re_opt = re.compile( rf'{name}(?=\:|$)')
+    return [k for k in options if re_opt.match(k)]
+
 def pop_extra_options(options, suffix):
+    """pop matching keys from options dict
+
+    :param options: source option dict (content will be modified)
+    :type options: dict
+    :param suffix: matching suffix
+    :type suffix: str
+    :return: popped options
+    :rtype: dict
+    """
     n = len(suffix)
     return {
         k[:-n]: options.pop(k)
@@ -589,6 +677,41 @@ def pop_extra_options(options, suffix):
     }
 
 
+def pop_extra_options_multi(options, suffix):
+    """pop regex matching keys from options dict and
+
+    :param options: source option dict (content will be modified)
+    :type options: dict
+    :param suffix: matching suffix regex expression with one group, capturing the (int) id
+    :type suffix: str
+    :return: dict of popped options with int id key
+    :rtype: str, dict(int, dict)
+
+    example:
+
+        pop_extra_options_multi({...},r'_in(\d+)$')
+
+    """
+
+    popped = {}
+
+    def match(name, v):
+        m = re.search(suffix, name)
+        if m:
+            k = name[: m.end()]
+            id = int(m[1])
+            if id in popped:
+                popped[id][k] = v
+            else:
+                popped[id] = {k: v}
+        return bool(m)
+
+    for o in (k for k, v in options.items() if match(k, v)):
+        options.pop(o)
+
+    return popped
+
+
 def bytes_to_ndarray(
     b,
     shape=None,
