@@ -1,3 +1,4 @@
+from copy import deepcopy
 import numpy as np
 
 from . import ffmpegprocess, utils, configure, FFmpegError, probe
@@ -50,7 +51,7 @@ def _run_read(
             *args,
             dtype=dtype,
             shape=shape,
-            capture_log=False if show_log else True,
+            capture_log=None if show_log else False,
             **kwargs,
         )
         if out.returncode:
@@ -196,13 +197,24 @@ def read(url, progress=None, show_log=None, **options):
     )
 
 
-def write(url, rate, data, progress=None, overwrite=None, show_log=None, **options):
+def write(
+    url,
+    rate_in,
+    data,
+    progress=None,
+    overwrite=None,
+    show_log=None,
+    two_pass=False,
+    pass1_omits=None,
+    pass1_extras=None,
+    **options,
+):
     """Write Numpy array to a video file
 
     :param url: URL of the video file to write.
     :type url: str
-    :param rate: frame rate in frames/second
-    :type rate: `float`, `int`, or `fractions.Fraction`
+    :param rate_in: frame rate in frames/second
+    :type rate_in: `float`, `int`, or `fractions.Fraction`
     :param data: video frame data 4-D array (frame x rows x cols x components)
     :type data: `numpy.ndarray`
     :param progress: progress callback function, defaults to None
@@ -213,6 +225,11 @@ def write(url, rate, data, progress=None, overwrite=None, show_log=None, **optio
     :param show_log: True to show FFmpeg log messages on the console,
                      defaults to None (no show/capture)
     :type show_log: bool, optional
+    :param two_pass: True to encode in 2-pass
+    :param pass1_omits: list of output arguments to ignore in pass 1, defaults to None
+    :type pass1_omits: seq(str), optional
+    :param pass1_extras: list of additional output arguments to include in pass 1, defaults to None
+    :type pass1_extras: dict(int:dict(str)), optional
     :param \\**options: FFmpeg options, append '_in' for input option names (see :doc:`options`)
     :type \\**options: dict, optional
     """
@@ -225,17 +242,29 @@ def write(url, rate, data, progress=None, overwrite=None, show_log=None, **optio
     configure.add_url(
         ffmpeg_args,
         "input",
-        *utils.array_to_video_input(rate, data=data, **input_options),
+        *utils.array_to_video_input(rate_in, data=data, **input_options),
     )
     configure.add_url(ffmpeg_args, "output", url, options)
 
-    ffmpegprocess.run(
+    configure.build_basic_vf(ffmpeg_args, configure.check_alpha_change(ffmpeg_args, -1))
+
+    kwargs = (
+        {
+            "pass1_omits": None if pass1_omits is None else [pass1_omits],
+            "pass1_extras": None if pass1_extras is None else [pass1_extras],
+        }
+        if two_pass
+        else {}
+    )
+
+    (ffmpegprocess.run_two_pass if two_pass else ffmpegprocess.run)(
         ffmpeg_args,
         input=data,
         stdout=stdout,
         progress=progress,
         overwrite=overwrite,
-        capture_log=False if show_log else None,
+        **kwargs,
+        capture_log=None if show_log else False,
     )
 
 
